/*************************************************************************
 
 Project2: shell : Solution

 A simple Unix shell

 *************************************************************************/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

#define TRUE 1
#define FALSE 0

#define MAX_INPUT_LENGTH 255
#define MAX_INPUT_WORDS 255

/**
 * Queries stdin for a \n terminated character array,
 * and returns an array of char* 's representing the sequence of
 * space delimited 'words' that the user typed, through the reference
 * paramter, and the number of words through the standard return.
 */

#define DELIMITERS " \t\n\r"

int getCommand( char*** inputWords )
{
  char inputLine[ MAX_INPUT_LENGTH ];
  int total = 0;
  char* token = NULL;

  *inputWords = NULL;
  /* read input */

  if (fgets(inputLine, MAX_INPUT_LENGTH, stdin) == NULL)
        return -1;

  *inputWords = (char**)malloc(MAX_INPUT_WORDS*sizeof(char*));
  
  /* parse out the tokens */

  total = 0;
  token = strtok( inputLine, DELIMITERS );
  while( token != NULL && total < MAX_INPUT_WORDS-1 )
    { /* read a word */
      (*inputWords)[total++] = strdup(token);
      token = strtok(NULL, DELIMITERS );
    }

/*
   Note note:  We add an extra entry, to null terminate the list, as
   required by exec.
   */
  (*inputWords)[total] = NULL;
  return total;
}



#define CHDIR_TEXT "cd"
#define EXIT_TEXT  "exit"


/**
 * Prints out the display prompt (A C macro, more efficient than
 * a function
 */
#define displayPrompt() { printf("CSE451Shell %% "); fflush(stdout); }

/**
 * Contains the functionality of attempting to change a directory,
 * and printing out any error messages that may result.
 *
 * parameter1:numTokens The number of tokens in tokens
 * parameter2:tokens: An array of char*'s, each such char* a token 
 *      from the user.
 */
void doChDir( int numTokens, char** tokens )
{
    if ( numTokens > 1 )
    { /* they gave us a path */
        int ret = chdir(tokens[1]);
        if ( ret != 0 )
            printf("Cannot change directory to \"%s\"\n", tokens[1] );
        else
            return;
    }     
    else { /* they just typed chdir */
            /* The environment variable $HOME contains the home directory */
            /* getenv() can be used to get the value of $HOME             */
          chdir(getenv("HOME"));
    }
}




int main(int argc, char** argv)
{
  char** tokens = NULL;
  int numTokens = 0;
  int i = 0;

  while( TRUE )
    {
      displayPrompt();
      numTokens = getCommand( &tokens );
      if (numTokens == -1) {
        printf("Error reading input line.\n");
        exit(1);
      }
      if ( tokens == NULL || numTokens == 0) /* no input, just return. */
         continue; 

      /* at least one token */
      if ( strcmp(tokens[0], CHDIR_TEXT) == 0 )
          doChDir( numTokens, tokens );
      else if ( strcmp(tokens[0], EXIT_TEXT) == 0 )
          exit(0);
      else { /* external command */
          pid_t pid = fork();
          int returnValue;

          if ( pid == 0 )
            { /* child  -- execute their command */

              int ret = execvp( tokens[0], tokens );
                  
              /* if we get here, an error has occurred */
              printf("Cannot find executable \"%s\"\n", tokens[0]);
              exit(1);
            }
          else
            { /* parent */
              if (wait(&returnValue) != -1) 
                    if (WIFEXITED(returnValue) && WEXITSTATUS(returnValue) != 0)
                        printf("Process %d returned error value %d\n", 
                                pid, WEXITSTATUS(returnValue));
            }
        }

      free( tokens );
    }
}


